# -*- coding:utf-8 -*-
import json
import os
import importlib.util
import importlib
import inspect
from importlib import import_module
from BearSki.CommonData import SkiGlobalData
from BearSki.utils.logger import SkiLogger

class Ski():
    class case():
        def __init__(self):
            # print('__case init__')
            self.logger=SkiLogger("BearSki.base")
            scd=SkiGlobalData()
        def __call__(self,func):
                def __deco(self,*arg,**kws):
                    # print("before %s called [%s],[%s]." % (func.__name__, arg,kws))
                    result=func(self,*arg,**kws)
                    # print("  after %s called [%s],[%s]." % (func.__name__, arg,kws))
                    return result
                return __deco

    class step():
        def __init__(self,keyword,*arg,**kws):
            self.logger=SkiLogger("BearSki.step")
            scd=SkiGlobalData()
            conf=scd.get_setting_data()
            full_modules=conf[keyword]
            self.result=self.__run(full_modules,*arg,**kws)
            # self.logger.debug(self.result)
        def __run(self,kw_path,*arg,**kws):
            try:
                modules=self.__getModules(kw_path)
            
            except Exception:
                self.logger.error("error,does not find  modules")
                return None
            
            fun_list=kw_path[len(modules)+1:].split('.')
            
            return self.__getObject(modules,fun_list)(*arg,**kws)

        def __getObject(self,modules,fun_list):
            
            obj= import_module(modules)
            child_obj=getattr(obj,fun_list[0])
            temp_cls_name=fun_list[0]
            for key in fun_list[1:]:
                if inspect.isclass(child_obj):
                    # print("this is a class")
                    temp_cls=SkiGlobalData().get_step_class_instance(temp_cls_name)
                    if temp_cls is None:
                        child_obj=child_obj()
                        SkiGlobalData().set_step_class_instance(temp_cls_name,child_obj)
                    else:
                        child_obj=temp_cls
                child_obj=getattr(child_obj,key)
                temp_cls_name=key
            return child_obj

        def __getModules(self,kw_path):
            # print(kw_path)
            if kw_path.find('.')==-1:
                if self.__ismodule(kw_path):
                    return modules
                else:
                    raise Exception("all error,does not find  modules")
            kw=kw_path.split('.')[-1]
            modules=kw_path[0:kw_path.rindex(kw)-1]  #rindex 为了应对报名重名 从后向前计算
            flag=self.__ismodule(modules)
            # print(flag)
            if flag:
                # print(modules)
                return modules
            else:
                # print(modules)
                return self.__getModules(modules)

        def __ismodule(self,module_name):

            # print("++++++ is module ++++")
            # print(module_name)
            # print("====== is module =====")
            module_spec=None
            try:
                module_spec = importlib.util.find_spec(module_name)
            except Exception as error:
                # print(error)
                logger.error("error",error)
                
                return False
            if module_spec is None:
                # print("Module :{} not found".format(module_name))
                # print("false")
                return False

            else:
                # print("Module:{} can be imported!".format(module_name))
                # print("true")
                return True
            

        


  
class command_parser(object):
  argument_array=[]
  def __init__(self):
    pass
  def add_argument(self,arg):
    self.argument_array.append(arg)
  def get_shot_arg(self):
    shot_arg=""
    for arg in self.argument_array:
      shot_arg=shot_arg+arg.shot_text+':'
    return shot_arg
  def get_long_arg(self):
    long_arg=[]
    for arg in self.argument_array:
      long_arg.append(arg.long_text)
    return long_arg
  def print_help_message(self):
    # print("Commands:")
    for arg in self.argument_array:
      message="  "+arg.shot_text
      for i in range(len(message),5):
        message=message+" "
      message=message+" --"+arg.long_text
      for i in range(len(message),30):
        message=message+" "
      # print(message+arg.helpmessage)

class base_command(object):
  shot_text=''
  long_text=''
  helpmessage=''
  def __init__(self,shot_text,long_text,helpmessage):
    self.shot_text=shot_text
    self.long_text=long_text
    self.helpmessage=helpmessage




import unittest
import logging
import time
import sys
from BearSki.utils.arguments import runArg
from BearSki.report.LocalReportRunner import LocalReportRunner
from BearSki.utils.logger import LoggerBaseConfig

class RunUnittest(object):

    def __init__(self):
        self.logger=logging.getLogger("BearSki.RunUnittest")
        
    def get_test_cases(self,dirpath,name="test_",isrunonecase=False):
        test_cases = unittest.TestSuite()
        # 测试用例使用"ski_"开头命名
        if isrunonecase:
            # 执行单条用例，顺序为 目录名 文件名 类名 方法名 中间"." 间隔 例如："testcase.debug_test_user.TestUserLogin.test_login"
            suite=unittest.TestLoader().loadTestsFromName(dirpath)
            test_cases.addTests(suite)
            return test_cases
        else:
            suites = unittest.defaultTestLoader.discover(dirpath, name+'*.py', top_level_dir=dirpath)
            for suite in suites:
                test_cases.addTests(suite)
            return test_cases

    def runTest(self,runArg):

        isrunonecase=False
        if runArg.mode=='onecase':
            isrunonecase=True
        casepath=runArg.case_path
        casename=runArg.case_name
        cases = self.get_test_cases(casepath,casename,isrunonecase)
        now = time.strftime("%Y-%m-%d %H_%M_%S")  # 报告生成时间
        if(runArg.report_mode=='text'):
            self.logger.info("开始执行测试,报告输出模式text")
            runner=unittest.TextTestRunner()
            runner.run(cases)
        elif(runArg.report_mode=='html'):
            LoggerBaseConfig()._htmlRunner_logset()
            self.logger.info("开始执行测试,报告输出模式html")
            lruner=LocalReportRunner()
            lruner.run(cases)
            self.logger.info("测试完成,报告输出模式html")



import os
import shutil
import sys
from BearSki.testproject_template import BasicTestProject
from BearSki.utils.logger import SkiLogger
logger=SkiLogger("Manager")
def create_project(projectname):
    BasicTestProject.create_testproject(projectname)
    
    






from setuptools import setup,find_packages
setup (
    name ='BearSki',
    version = '1.2.3',
    packages = find_packages('src'),  # 包含所有src中的包
    package_dir = {'':'src'},   # 告诉distutils包都在src下
    package_data = {
        # 任何包中含有.txt文件，都包含它
        '': ['*.txt'],
        # 包含demo包data文件夹中的 *.dat文件
        'BearSki': ['data/*.dat'],
    },
    description = 'AutoTest BaseFramework',
    author = 'sirius',
    url = 'https://www.agavetest.info',
    license ='MIT',
    py_modules=['hello']
)